<?php

namespace OrganizationsManagement\Service;
use \Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\InstanceProvider;



class FileSendingSolicitationDelphi
{

    const TMP_PATH_OM = '/tmp/om_file_send_solicitation/';

    public function findFileSendingSolicitationDelphi($conditions, $page, $pageSize)
    {
        $filters = Utilities::createQueryFiltered($conditions);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return InstanceProvider::getRepository()->getSoliserv()->findFileSendingSolicitationDelphi($filters, $limit, $offset);
    }

    public function fileSendingSolicitationDownload($rows)
    {
        Utilities::deleteDirectory(self::TMP_PATH_OM);
        $logFaseTram = $this->getLogfasetram($rows);
        $error = $this->validateIfAllValuesAreNullOfSoliServe($logFaseTram);
        if(!empty($error)) {
            return array('status' => false, 'errorType' => 'VALIDATION_NRSOLISERV', 'data' => $error);
        }

        if(!file_exists(self::TMP_PATH_OM)) {
            $this->createDirectories();
        }

        $error = $this->preparePackageZip($logFaseTram);
        if(!empty($error)) {
            return array('status' => false, 'errorType' => 'VALIDATION_NRSOLISERV', 'data' => $error);
        }

        if(empty(Utilities::dirToArray(self::TMP_PATH_OM . 'tozip/'))) {
            return array('status' => false, 'errorType' => 'EMPTY_TOZIP_DIRECTORY');
        }

        Utilities::toZipDir(self::TMP_PATH_OM . 'tozip/', self::TMP_PATH_OM . 'package.zip');
        return array('status' => true);
    }

    public function getLogfasetram($rows)
    {
        $cdFaseTram = Utilities::getCdFaseTramFilterParameter();
        $nrSoliServiConcat = '';
        foreach ($rows as $row) {
            $nrSoliServiConcat .= $row['NRSOLI_ENV_ARQ'] . '_';
        }

        return InstanceProvider::getRepository()->getLogfasetram()->findLogFaseTram($nrSoliServiConcat, $cdFaseTram);
    }

    public function createDirectories()
    {
        mkdir(self::TMP_PATH_OM, 0775, true);
        mkdir(self::TMP_PATH_OM . 'ziped/', 0775, true);
        mkdir(self::TMP_PATH_OM . 'tozip/', 0775, true);
        mkdir(self::TMP_PATH_OM . 'extracted/', 0775, true);
    }

    public function preparePackageZip($logFaseTram)
    {
        $allowedFiles = Utilities::getAllowedFilesToBeSentParameterArray();

        $errorControl = array();
        $successControl = array();
        $zip = new \ZipArchive();

        foreach($logFaseTram as $log) {
            $pathSource = Utilities::getFilePathToBeSentDelphiParameter();
            $pathSource .= $log['NRSOLISERV'] . '/' . $log['DTALFASETRAM'];
            if(is_dir($pathSource) && !empty(Utilities::dirToArray($pathSource)) ) {
                $files = Utilities::dirToArray($pathSource);
                foreach ($files as $file) {
                    $isFileExists = Utilities::strpos_arr($file, $allowedFiles);
                    if ($isFileExists) {

                        $successControl[] = $log['NRSOLISERV'];
                        $key = array_search($log['NRSOLISERV'], $errorControl);
                        if($key !== false) {
                            unset($errorControl[$key]);
                        }

                        copy($pathSource . '/' . $file, self::TMP_PATH_OM . 'ziped/' . $file);
                        if ($zip->open(self::TMP_PATH_OM . 'ziped/' . $file) === true) {
                            $zip->extractTo(self::TMP_PATH_OM . 'extracted/');
                            $zip->close();
                            Utilities::copyDir(self::TMP_PATH_OM . 'extracted/', self::TMP_PATH_OM . 'tozip/');
                        }
                        Utilities::deleteAllFilesInFolder(self::TMP_PATH_OM . 'extracted');
                        Utilities::deleteAllFilesInFolder(self::TMP_PATH_OM . 'ziped');
                    } else {
                        if(!in_array($log['NRSOLISERV'], $successControl) && !in_array($log['NRSOLISERV'], $errorControl)) {
                            $errorControl[] = $log['NRSOLISERV'];
                        }
                    }
                }
            } else {
                if(!in_array($log['NRSOLISERV'], $successControl) && !in_array($log['NRSOLISERV'], $errorControl)) {
                    $errorControl[] = $log['NRSOLISERV'];
                }
            }
        }
        $errorControl = array_values($errorControl);
        return $errorControl;
    }

    public function downLoad()
    {
        $filename = self::TMP_PATH_OM . 'package.zip';
        if (file_exists($filename)) {
            header('Content-Type: application/zip');
            header("Content-Disposition: attachment; filename=" . "package.zip");
            header('Content-Length: ' . filesize($filename));
            readfile($filename);
            unlink($filename);
            Utilities::deleteDirectory(self::TMP_PATH_OM);
        } else {
            throw new \Exception('Arquivo não encontrado.');
        }
    }

    public function validateIfAllValuesAreNullOfSoliServe($values)
    {
        $statusControlError = array();
        $statusControlSuccess = array();

        foreach ($values as $register) {
            if(empty($register['DTALFASETRAM'])) {
                if(!in_array($register['NRSOLISERV'], $statusControlSuccess) && !in_array($register['NRSOLISERV'], $statusControlError)) {
                    $statusControlError[] = $register['NRSOLISERV'];
                }
            } else {
                $statusControlSuccess[] = $register['NRSOLISERV'];
                $key = array_search($register['NRSOLISERV'], $statusControlError);
                if($key !== false) {
                    unset($statusControlError[$key]);
                }
            }
        }
        $statusControlError= array_values($statusControlError);
        return $statusControlError;
    }


}